"""
@File    :   semantic_segmentation.py
@Time    :   2024/06/30 00:25:55
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

import random
import os
import os.path as osp
from glob import glob

import cv2
from PIL import Image
import numpy as np
import torch
from torch.utils.data import Dataset, DataLoader
from torchvision import datasets, transforms
from torchvision.transforms import (
    Compose,
    Resize,
    ToTensor,
    Normalize,
    InterpolationMode,
)

from rlmc.data.utils import mask2onehot

__all__ = ["SemanticSegmentationDataset"]


class SemanticSegmentationDataset(Dataset):
    def __init__(self, configs, data_dir):
        self.args = configs
        self.data_dir = data_dir
        self.imgs = glob(osp.join(data_dir, "images", "*.jpg"))
        self.input_size = configs.dataset.input_size
        self.transform_img = transforms.Compose(
            [
                ToTensor(),  # 转为torch.Tensor,变0~1
                Resize(
                    (self.input_size.height, self.input_size.width),
                    interpolation=InterpolationMode.BICUBIC,
                ),
                Normalize(
                    self.args.dataset.transformNormalize.mean,
                    self.args.dataset.transformNormalize.std,
                ),
            ]
        )

    def __len__(
        self,
    ):
        return len(self.imgs)

    def __getitem__(self, item):
        file_name = osp.splitext(osp.basename(self.imgs[item]))[0]
        img = Image.open(self.imgs[item])
        img = self.transform_img(img)
        mask = Image.open(osp.join(self.data_dir, "masks", file_name + ".png"))
        mask = np.array(mask)
        mask[np.where(mask == 255)] = 0  # 剔除voc的255边

        mask = cv2.resize(
            mask,
            (self.input_size.width, self.input_size.height),
            interpolation=cv2.INTER_NEAREST,
        )

        mask = torch.from_numpy(mask).long()
        mask = torch.unsqueeze(mask, dim=0)
        # mask_=mask2onehot(mask,self.args.dataset.num_classes)

        mask_ = torch.zeros(
            self.args.dataset.num_classes, self.input_size.height, self.input_size.width
        )
        mask_ = mask_.scatter(0, mask, 1)  # scatter(dim , label ,1 )  # one-hot

        mask_ = mask_.float()
        return img, mask_



"""
@File    :   take_photo.py
@Time    :   2024/07/20 21:22:38
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import argparse
import os
import os.path as osp

import cv2
import rich
from rich.console import Console

parser = argparse.ArgumentParser("take photo")
parser.add_argument("-o", "--output_dir", type=str, required=True)
parser.add_argument("-w", "--width", type=int, default=0)
parser.add_argument("-ht", "--height", type=int, default=0)
args = parser.parse_args()

rc = Console()


def main() -> None:

    rc.print(
        "[green]###  f: 拍摄图片",
    )  # style=rich.style.Style(color='white', bgcolor='white')
    rc.print("[green]###  esc: 退出")

    cap = cv2.VideoCapture(
        0,
    )  # cv2.CAP_DSHOW
    width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
    height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    rc.print(f"[green]###  摄像头分辨率 w x h: {width} x {height}")
    rc.rule("[bold green]Take Photo")
    index = 0
    while True:
        ret, frame = cap.read()

        frame = cv2.flip(frame, 1, dst=None)
        if args.width > 0 and args.height > 0:
            frame = cv2.resize(
                frame, (args.width, args.height), interpolation=cv2.INTER_LINEAR
            )
        cv2.imshow("capture", frame)
        input = cv2.waitKey(1) & 0xFF

        if input == ord("f"):
            cv2.imwrite("%s/%d.jpg" % (args.output_dir, index), frame)
            print("%s: %d 张图片" % (args.output_dir, index))
            index += 1
        elif input == 27:
            break
        else:
            pass

    cap.release()
    cv2.destroyAllWindows()


if __name__ == "__main__":
    main()



# 注意余额或key变更，插件调用，在百度智能云更改插件，“插件编排”，上线下线，在插件编辑中重新上传知识库文件。目前问题是章节不会区分，基本只会按整个文件进行理解搜索，部分理解可能也不太对，需要对结果校对
# https://demo.ragflow.io/
import argparse
import os
import pyttsx3

import qianfan

parser = argparse.ArgumentParser("qianfan")
parser.add_argument(
    "-a",
    "--access_key",
    required=True,
)
parser.add_argument(
    "-s",
    "--secret_key",
    required=True,
)
parser.add_argument(
    "-m",
    "--model",
    default="ERNIE-Bot-turbo",  # ERNIE-4.0-8K ChatLaw
    help="ERNIE-4.0-8K, ChatLaw, ERNIE-Bot-turbo...",
)

parser.add_argument(
    "-sp",
    "--is_speak",
    action="store_true",
)

args = parser.parse_args()


# 【推荐】使用安全认证AK/SK鉴权，通过环境变量初始化认证信息
# 替换下列示例中参数，安全认证Access Key替换your_iam_ak，Secret Key替换your_iam_sk
os.environ["QIANFAN_ACCESS_KEY"] = args.access_key
os.environ["QIANFAN_SECRET_KEY"] = args.secret_key


def main() -> None:
    chat_comp = qianfan.ChatCompletion()

    msgs = qianfan.Messages()
    while True:
        msgs.append(input("master:"))
        resp = chat_comp.do(model=args.model, messages=msgs)
        print("assistant:", resp["result"])
        if args.is_speak:
            pyttsx3.speak(resp["result"])
        msgs.append(resp)


if __name__ == "__main__":
    main()



"""
@File    :   opt.py
@Time    :   2024/06/30 00:50:09
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

from torch import optim

__all__ = ["optimizer"]


optimizer = {
    "RMSprop": optim.RMSprop,
    "SGD": optim.SGD,
    "Adam": optim.Adam,
    "AdamW": optim.AdamW,
}



"""
@File    :   __init__.py
@Time    :   2024/06/19 13:38:17
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

__version__ = "1.1.42"

import sys

if "." not in sys.path:
    sys.path.append(".")

from rlmc.utils.logger import Logger
from rlmc.utils.mc import MetaList
from rlmc.utils.register import Register
from rlmc.utils.sudict import SuDict
from rlmc.utils.multiprocess import MultiProcess
from rlmc.utils.multithread import MultiThread
from rlmc.utils.asynctask import (
    AsyncTasks,
    AsyncProducerConsumer,
    AsyncProducerConsumerTriple,
)

from rlmc.utils.downloadscript import HfDownload, MsDownload, AutodlDownload

from rlmc.fileop.utils import FindContent, DatasetSplit

from rlmc.utils.login_module import user_login   # decrorator

from rlmc.utils.coroutine import Abstract_ManMachineChat  # abstract class

from rlmc.fileop import file_processor   # 

from rlmc.configs import (
    user_setting,
    model_download_urls,
    dataset_download_urls,
    common_urls,
)
from rlmc.resource import condarc, pip, cuda


reg = Register()
reg.register(MetaList)
reg.register(Logger)
reg.register(SuDict)
reg.register(MultiProcess)
reg.register(MultiThread)
reg.register(AsyncTasks)
reg.register(AsyncProducerConsumer)
reg.register(AsyncProducerConsumerTriple)
reg.register(Abstract_ManMachineChat)
reg.register(HfDownload)
reg.register(MsDownload)
reg.register(AutodlDownload)
reg.register(FindContent)
reg.register(DatasetSplit)
reg.register(file_processor)
reg.register(user_login)

cfg = SuDict()
cfg["common_urls"] = SuDict(common_urls)
cfg["user_setting"] = SuDict(user_setting)
cfg["model_download_urls"] = SuDict(model_download_urls)
cfg["dataset_download_urls"] = SuDict(dataset_download_urls)
cfg["source"] = SuDict({"pip": pip, "condarc": condarc, "cuda": cuda})


__all__ = [
    "__version__",
    "Logger",
    "MetaList",
    "Register",
    "SuDict",
    "MultiProcess",
    "MultiThread",
    "AsyncTasks",
    "AsyncProducerConsumer",
    "AsyncProducerConsumerTriple",
    "Abstract_ManMachineChat",
    "HfDownload",
    "MsDownload",
    "AutodlDownload",
    "FindContent",
    "DatasetSplit",
    "file_processor",
    "reg",
    "cfg",
]


if __name__ == "__main__":
    print(reg.keys())
    yamlobj = reg["file_processor"]("rlmc/configs/script_paths.yaml")
    yaml_content = yamlobj.data
    print(yaml_content)



"""
@File    :   downloadscript.py
@Time    :   2024/06/18 22:37:02
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import Any, NoReturn, Union, Callable, Literal, Iterable, List, Dict
import os
import os.path as osp
import shutil

from rlmc.configs import user_setting, model_download_urls, dataset_download_urls
from rlmc.utils.logger import Logger


# os.environ["XDG_CACHE_HOME"] = user_setting["XDG_CACHE_HOME"]
# os.environ["MODELSCOPE_CACHE"] = f"{user_setting['XDG_CACHE_HOME']}/modelscope"

os.environ["HF_ENDPOINT"] = user_setting["HF_ENDPOINT"]

from huggingface_hub import snapshot_download, login


__all__ = ["HfDownload", "MsDownload", "AutodlDownload"]


logger = Logger(__name__, level=Logger.DEBUG)


class HfDownload:
    """how to use
    if __name__=='__main__':
        repo_id = "meta-llama/Llama-2-7b-hf"
        repo_type = "model"
        local_dir=f"{user_setting['XDG_CACHE_HOME']}/{repo_id.split('/')[-1]}"
        hf_download=HfDownload(repo_id,local_dir,repo_type=repo_type, is_login=False)
        hf_download.run()
    """

    def __init__(
        self,
        repo_id: str,
        local_dir: str,
        local_dir_use_symlinks: bool = False,
        is_login: bool = False,
        hf_token: str = "hf_JiIBOPMbYRjssOTAsdWo",
        repo_type: Literal["model", "dataset"] = "model",
        ignore_patterns: List[str] = ["*.h5", "*safetensors", "*msgpack"],
        force_download: bool = False,
        resume_download: bool = True,  # 断点续传
        etag_timeout: int = 1200,  # 超时阈值
    ) -> None:
        if is_login:
            login(token=hf_token)
        os.makedirs(local_dir, exist_ok=True)
        self.repo_id = repo_id
        self.local_dir = local_dir
        self.local_dir_use_symlinks = local_dir_use_symlinks
        self.repo_type = repo_type
        self.ignore_patterns = ignore_patterns
        self.force_download = force_download
        self.resume_download = resume_download
        self.etag_timeout = etag_timeout

    def run(self) -> None:
        logger.info(f"************ Start downloading {self.repo_id} ************")
        snapshot_download(
            repo_id=self.repo_id,
            cache_dir=self.local_dir,
            local_dir=self.local_dir,
            local_dir_use_symlinks=self.local_dir_use_symlinks,
            ignore_patterns=self.ignore_patterns,
            force_download=self.force_download,
            resume_download=self.resume_download,
            etag_timeout=self.etag_timeout,
        )

        logger.info(f"************ {self.repo_id} Download finish ************")
        logger.info(f"Models saved in {self.local_dir}")


class MsDownload:
    """how to use
    if __name__=='__main__':
        repo_id='skyline2006/llama-7b'
        ms_download=MsDownload(repo_id)
        ms_download.run()
    """

    def __init__(
        self,
        repo_id: str,
        cache_dir: str='./MscopeCache',
        ignore_file_pattern: List[str] = ["*.h5", "*safetensors", "*msgpack"],
    ) -> None:
        self.repo_id = repo_id
        self.cache_dir = cache_dir
        self.ignore_file_pattern = ignore_file_pattern

    def run(self) -> None:
        from modelscope import snapshot_download as ms_snapshot_download

        logger.info(f"************ Start downloading {self.repo_id} ************")
        model_dir = ms_snapshot_download(
            self.repo_id,
            cache_dir=self.cache_dir,
            ignore_file_pattern=self.ignore_file_pattern,
        )
        logger.info(f"************ {self.repo_id} Download finish ************")
        logger.info(f"Models saved in {model_dir}")


class AutodlDownload:
    def __init__(self, repo_id: str, target_directory: str) -> None:
        self.repo_id = repo_id
        self.target_directory = target_directory

    def run(self) -> None:
        import codewithgpu as cg

        logger.info(f"************ Start downloading {self.repo_id} ************")
        cg.model.download(self.repo_id, target_directory=self.target_directory)
        logger.info(f"************ {self.repo_id} Download finish ************")
        # logger.info(f'Models saved in {self.local_dir}')


if __name__ == "__main__":
    # repo_id = "meta-llama/Llama-2-7b-hf"
    # repo_type = "model"
    # local_dir=f"{user_setting['XDG_CACHE_HOME']}/{repo_id.split('/')[-1]}"
    # hf_download=HfDownload(repo_id,local_dir,repo_type=repo_type, is_login=False)
    # hf_download.run()

    repo_id = "skyline2006/llama-7b"
    ms_download = MsDownload(repo_id)
    ms_download.run()



"""
@File    :   spider.py
@Time    :   2024/07/24 22:45:48
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

import asyncio
import random
import time
import contextvars
import functools
import os
import os.path as osp
import re
from typing import Callable, List, Tuple, Dict, Union, Optional, Generator

import requests
import aiohttp
import aiofiles
from lxml import etree


from rlmc.utils.logger import Logger


python_version = f"{sys.version_info.major}.{sys.version_info.minor}"


# ---------python3.8 没有asyncio.to_thread，此处支持python3.8--------*
async def to_thread(func, *args, **kwargs):
    loop = asyncio.get_running_loop()
    ctx = contextvars.copy_context()
    func_call = functools.partial(ctx.run, func, *args, **kwargs)
    return await loop.run_in_executor(None, func_call)


try:
    import asyncio.to_thread as async_to_thread
except:
    async_to_thread = to_thread

# ---------python3.8 没有asyncio.to_thread，此处支持python3.8--------*

__all__ = ["Spider"]
logger = Logger(__name__, level=Logger.DEBUG)


class Spider:
    """how to use"""

    _default = {
        "headers": [
            {
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36"
            }
        ],
        "verify": False,
        "proxy": [],  # ['58.246.58.150:9002', '117.74.65.215:9080', '88.132.253.187:80']
        "timeout": 1000,
        "encoding": "gbk",  # utf-8
    }

    def __init__(
        self,
        url_list: List,
        producer_semaphore_num: int,
        consumer_func: Callable,
        consumer_semaphore_num: int,
    ):
        self.__dict__.update(self._default)
        self.url_list = url_list
        self.producer_semaphore_num = producer_semaphore_num
        self.consumer_semaphore_num = consumer_semaphore_num
        self.consumer_func = consumer_func
        self.result = {}

    @classmethod
    def _get_contents(cls, res: requests.Response) -> Optional[str]:
        """
        从response decode内容，编码处理
        """
        if res is None:
            decodedata = None
        elif res.status_code == 200:
            decodedata = res.content.decode(cls._default["encoding"])
            print(cls._default["encoding"])
        else:
            decodedata = None
        return decodedata

    @classmethod
    def _gen_next_page(
        cls, host_url: str, start_url: str, end_url: str, next_page_xpath: str
    ) -> Generator:
        current_url = start_url
        while True:
            yield current_url

            if cls._default["proxy"]:
                response = requests.get(
                    current_url,
                    timeout=cls._default["timeout"],
                    headers=cls._default["headers"][0],
                    verify=cls._default["verify"],
                    proxies=cls._default["proxy"],
                )
            else:
                response = requests.get(
                    current_url,
                    timeout=cls._default["timeout"],
                    headers=cls._default["headers"][0],
                    verify=cls._default["verify"],
                )
            decodedata = cls._get_contents(response)
            data = etree.HTML(decodedata)

            #### 看情况改 ####
            # 同步，next_page   1、测试next_page_xpath，2、再测试content_xpath（可多个），3、写记录保存代码

            # content_xpath = '' #从current url内容获取需要信息
            # content=data.xpath(content_xpath)

            if current_url == end_url:
                break

            elem = data.xpath(next_page_xpath)  # 从current url内容获取下一页
            # elem转字符查看结构： etree.tostring(elem[0])
            next_url = host_url + elem[0].attrib["href"]
            print(next_url)
            #### 看情况改 ####

            current_url = next_url

    @classmethod
    def get_all_urls(
        cls, host_url: str, start_url: str, end_url: str, xpath: str
    ) -> List:
        generator = cls._gen_next_page(host_url, start_url, end_url, xpath)
        return [url for url in generator]

    async def producer(
        self,
        name: str,
        queue: asyncio.Queue,
        elem_idx: int,
        url: str,
        semaphore: asyncio.Semaphore,
        session: aiohttp.ClientSession,
    ):
        async with semaphore:
            async with session.get(url) as response:
                print(response.status)
                data = await response.text(self.encoding)
                content = await response.read()
                resp = {"text": data, "content": content}
                await queue.put((elem_idx, url, resp))
                logger.info(f"{name} is produced item: {url}")

    async def consumer(self, name: str, queue: asyncio.Queue):
        #### 看情况改 ####
        # 异步处理只要改consumer即可
        #### 看情况改 ####
        while True:
            elem_idx, url, item = await queue.get()
            res = await asyncio.create_task(
                async_to_thread(self.consumer_func, url, item)
            )

            queue.task_done()
            self.result[elem_idx] = (url, res)
            logger.info(f"{name} is working on item: {url}")

    async def run(self):
        queue = asyncio.Queue()
        semaphore = asyncio.Semaphore(self.producer_semaphore_num)

        conn = aiohttp.TCPConnector(ssl=self.verify, limit=None)
        timeout = aiohttp.ClientTimeout(total=self.timeout)
        async with aiohttp.ClientSession(
            headers=self.headers[0], connector=conn, timeout=timeout
        ) as session:
            producer_tasks = []
            for idx, url in enumerate(self.url_list):
                task = asyncio.create_task(
                    self.producer(
                        f"producer-worker-{idx}", queue, idx, url, semaphore, session
                    )
                )
                producer_tasks.append(task)

            tasks = []
            for i in range(self.consumer_semaphore_num):
                task = asyncio.create_task(self.consumer(f"consumer-worker-{i}", queue))
                tasks.append(task)

            # Wait until all worker tasks are cancelled.
            await asyncio.gather(*producer_tasks, return_exceptions=True)

            # Wait until the queue is fully processed.
            started_at = time.monotonic()
            await queue.join()
            total_slept_for = time.monotonic() - started_at

            # Cancel our worker tasks.
            for task in tasks:
                task.cancel()
            return tasks

    async def main(self):  # 重要，不套一个await会没有结果
        await self.run()


def sync_co_task(url: str, resp: Dict):
    print("--------------------------")
    # print(type(resp['text']))
    data = etree.HTML(resp["text"])
    elem = data.xpath("//div")
    # print(etree.tostring(data))
    # print(elem[20].text)
    content = ""
    for ele in elem:
        if ele is not None:  # 重要
            if ele.text is not None:  # 重要
                content += ele.text

    return content


if __name__ == "__main__":
    ##同步 next page下一页##
    # host_url = "https://www..com"
    # start_url = "https://www..com/2021n/12/19118.html"
    # end_url = "https://www..com/2021n/12/19118_4.html"
    # next_page_xpath = "//a[contains(text(),'下一页')]"
    # urls = Spider.get_all_urls(host_url, start_url, end_url, next_page_xpath)
    # print(urls)
    ##同步 next page下一页##

    ##异步 已知url列表##
    start = time.time()
    task_list = [
        "https://www..com/2021n/12/19118_4.html",
    ]
    # task_list=['https://www..info/266/266564/76434073.html']
    spider = Spider(
        task_list,
        producer_semaphore_num=3,
        consumer_func=sync_co_task,
        consumer_semaphore_num=3,
    )

    main = spider.main()
    asyncio.run(main)
    print(spider.result)
    print(time.time() - start)



"""
@File    :   asynctask.py
@Time    :   2024/06/21 13:08:38
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

import asyncio
import contextvars
import functools
import time
import random
from abc import ABC, abstractmethod
from typing import Callable, Union, Dict, List, Any


from rlmc.utils.logger import Logger


python_version = f"{sys.version_info.major}.{sys.version_info.minor}"


# ---------python3.8 没有asyncio.to_thread，此处支持python3.8--------*
async def to_thread(func, *args, **kwargs):
    loop = asyncio.get_running_loop()
    ctx = contextvars.copy_context()
    func_call = functools.partial(ctx.run, func, *args, **kwargs)
    return await loop.run_in_executor(None, func_call)


try:
    import asyncio.to_thread as async_to_thread
except:
    async_to_thread = to_thread

# ---------python3.8 没有asyncio.to_thread，此处支持python3.8--------*

__all__ = ["AsyncTasks", "AsyncProducerConsumer", "AsyncProducerConsumerTriple"]

logger = Logger(__name__, level=Logger.DEBUG)


class AsyncTasks:
    """how to use
    if __name__ == "__main__":
        start = time.time()
        asynctasks = AsyncTasks(semaphore_num=2)

        task1 = asynctasks.create_task(sync_task, 2, 9)
        task2 = asynctasks.create_task(sync_task, 2, 9)
        task3 = asynctasks.create_task(sync_task, 3, 6)

        main = asynctasks.main(*[task1, task2, task3])
        res = asyncio.run(main)
        print(res)
        print(time.time() - start)
    """

    def __init__(self, semaphore_num: int) -> None:
        self.semaphore_num = semaphore_num

    async def _async_wrapper(self, sync_func: Callable, *args, **kwargs):
        coroutine = await async_to_thread(sync_func, *args, **kwargs)
        return coroutine

    async def _semaphore(self, semaphore: asyncio.Semaphore, task: asyncio.Task):
        # 控制异步并发数，with semaphore要套每个task
        async with semaphore:
            return await task

    async def create_task(self, sync_func: Callable, *args, **kwargs) -> asyncio.Task:
        task = await asyncio.create_task(
            self._async_wrapper(sync_func, *args, **kwargs)
        )
        return task

    async def main(self, *tasks) -> Any:
        # semaphore初始化要在gather前
        semaphore = asyncio.Semaphore(self.semaphore_num)
        tasks = [self._semaphore(semaphore, task) for task in tasks]
        res = await asyncio.gather(*tasks)
        return res


class AsyncProducerConsumer:
    """how to use
    if __name__ == "__main__":
        start = time.time()
        task_list = [2, 0.2, 0, 1, 1, 4, 0.7]
        asyncproductorconsumer = AsyncProducerConsumer(
            task_list,
            producer_func=sync_pro_task,
            producer_semaphore_num=7,
            consumer_func=sync_co_task,
            consumer_semaphore_num=7,
        )

        main = asyncproductorconsumer.main()
        asyncio.run(main)
        print(asyncproductorconsumer.result)
        print(time.time() - start)
    """

    def __init__(
        self,
        element_list,
        producer_func,
        producer_semaphore_num,
        consumer_func,
        consumer_semaphore_num,
    ):
        self.producer_semaphore_num = producer_semaphore_num
        self.consumer_semaphore_num = consumer_semaphore_num
        self.element_list = element_list
        self.producer_func = producer_func
        self.consumer_func = consumer_func
        self.result = {}

    async def producer(self, name, queue: asyncio.Queue, elem_idx, elem, semaphore):
        async with semaphore:
            coroutine = async_to_thread(self.producer_func, elem)
            item = await asyncio.create_task(coroutine)
            await queue.put((elem_idx, elem, item))
            logger.info(f"{name} is produced item: {item}")

    async def consumer(self, name, queue: asyncio.Queue):
        while True:
            elem_idx, elem, item = await queue.get()
            res = await asyncio.create_task(async_to_thread(self.consumer_func, item))
            queue.task_done()
            self.result[elem_idx] = (elem, res)
            logger.info(f"{name} is working on item: {item}")

    async def main(self):
        queue = asyncio.Queue()
        semaphore = asyncio.Semaphore(self.producer_semaphore_num)

        producer_tasks = []
        for idx, elem in enumerate(self.element_list):
            task = asyncio.create_task(
                self.producer(f"producer-worker-{idx}", queue, idx, elem, semaphore)
            )
            producer_tasks.append(task)

        # total_sleep_time = 0
        # for _ in range(20):
        #     sleep_for = random.uniform(0.05, 1.0)
        #     total_sleep_time += sleep_for
        #     queue.put_nowait(sleep_for)

        tasks = []
        for i in range(self.consumer_semaphore_num):
            task = asyncio.create_task(self.consumer(f"consumer-worker-{i}", queue))
            tasks.append(task)

        # Wait until all worker tasks are cancelled.
        await asyncio.gather(*producer_tasks, return_exceptions=True)

        # Wait until the queue is fully processed.
        started_at = time.monotonic()
        await queue.join()
        total_slept_for = time.monotonic() - started_at

        # Cancel our worker tasks.
        for task in tasks:
            task.cancel()
        return tasks


class AsyncProducerConsumerTriple:
    """how to use
    if __name__ == "__main__":
        start = time.time()
        task_list = [1, 0.2, 0, 1, 1, 1, 0.7,0.8,0.9,1,1]
        asyncproductorconsumer = AsyncProducerConsumerTriple(
            task_list,
            producer_func=sync_pro_task,
            producer_semaphore_num=14,
            middle_func=sync_co_task,
            middle_semaphore_num=10,
            consumer_func=sync_co_task,
            consumer_semaphore_num=9,
        )

        main = asyncproductorconsumer.main()
        asyncio.run(main)
        print(asyncproductorconsumer.result)
        print(time.time() - start)
    """

    def __init__(
        self,
        element_list,
        producer_func,
        producer_semaphore_num,
        middle_func,
        middle_semaphore_num,
        consumer_func,
        consumer_semaphore_num,
    ):
        self.producer_semaphore_num = producer_semaphore_num
        self.consumer_semaphore_num = consumer_semaphore_num
        self.middle_func = middle_func
        self.middle_semaphore_num = middle_semaphore_num
        self.element_list = element_list
        self.producer_func = producer_func
        self.consumer_func = consumer_func
        self.result = {}

    async def producer(self, name, queue, elem_idx, elem, semaphore):
        async with semaphore:
            coroutine = async_to_thread(self.producer_func, elem)
            item = await asyncio.create_task(coroutine)
            await queue.put((elem_idx, elem, item))
            logger.info(f"{name} is produced item: {item}")

    async def consumer(self, name, queue):
        while True:
            elem_idx, elem, item = await queue.get()
            res = await asyncio.create_task(async_to_thread(self.consumer_func, item))
            queue.task_done()
            self.result[elem_idx] = (elem, res)
            logger.info(f"{name} is working on item: {item}")

    async def middle(self, name, queue_from_producer, queue_to_consumer):
        while True:
            elem_idx, elem, item = await queue_from_producer.get()
            item = await asyncio.create_task(async_to_thread(self.middle_func, item))
            queue_from_producer.task_done()
            await queue_to_consumer.put((elem_idx, elem, item))
            logger.info(f"{name} is middle working on item: {item}")

    async def main(self):
        queue_producer = asyncio.Queue()
        queue_middle = asyncio.Queue()
        semaphore = asyncio.Semaphore(self.producer_semaphore_num)

        producer_tasks = []
        for idx, elem in enumerate(self.element_list):
            task = asyncio.create_task(
                self.producer(
                    f"producer-worker-{idx}", queue_producer, idx, elem, semaphore
                )
            )
            producer_tasks.append(task)

        tasks_middle = []
        for i in range(self.middle_semaphore_num):
            task = asyncio.create_task(
                self.middle(f"middle-worker-{i}", queue_producer, queue_middle)
            )
            tasks_middle.append(task)

        tasks = []
        for i in range(self.consumer_semaphore_num):
            task = asyncio.create_task(
                self.consumer(f"consumer-worker-{i}", queue_middle)
            )
            tasks.append(task)

        # Wait until all worker tasks are cancelled.
        await asyncio.gather(*producer_tasks, return_exceptions=True)

        # Wait until the queue is fully processed.
        started_at = time.monotonic()
        await queue_producer.join()
        await queue_middle.join()
        total_slept_for = time.monotonic() - started_at

        # Cancel our worker tasks.
        for task in tasks:
            task.cancel()
        return tasks


# 一个原同步函数
def sync_pro_task(interval):
    time.sleep(interval)
    return interval


def sync_co_task(interval):
    time.sleep(interval)
    return interval + 1


def sync_task(interval, num):
    time.sleep(interval)
    return f"{interval}_{num}"


if __name__ == "__main__":
    # start = time.time()
    # asynctasks = AsyncTasks(semaphore_num=2)

    # task1 = asynctasks.create_task(sync_task, 2, 9)
    # task2 = asynctasks.create_task(sync_task, 2, 9)
    # task3 = asynctasks.create_task(sync_task, 3, 6)

    # main = asynctasks.main(*[task1, task2, task3])
    # res = asyncio.run(main)
    # print(res)
    # print(time.time() - start)

    # start = time.time()
    # task_list = [2, 0.2, 0, 1, 1, 4, 0.7]
    # asyncproductorconsumer = AsyncProducerConsumer(
    #     task_list,
    #     producer_func=sync_pro_task,
    #     producer_semaphore_num=7,
    #     consumer_func=sync_co_task,
    #     consumer_semaphore_num=7,
    # )

    # main = asyncproductorconsumer.main()
    # asyncio.run(main)
    # print(asyncproductorconsumer.result)
    # print(time.time() - start)

    start = time.time()
    task_list = [1, 0.2, 0, 1, 1, 1, 0.7, 0.8, 0.9, 1, 1]
    asyncproductorconsumer = AsyncProducerConsumerTriple(
        task_list,
        producer_func=sync_pro_task,
        producer_semaphore_num=14,
        middle_func=sync_co_task,
        middle_semaphore_num=10,
        consumer_func=sync_co_task,
        consumer_semaphore_num=9,
    )

    main = asyncproductorconsumer.main()
    asyncio.run(main)
    print(asyncproductorconsumer.result)
    print(time.time() - start)



import random

from PIL import Image
import numpy as np
import torch
from torch.utils.data import Dataset, DataLoader
from torchvision import datasets, transforms, InterpolationMode

random.seed(123)
np.random.seed(123)
torch.manual_seed(123)
if torch.cuda.is_available():
    torch.cuda.manual_seed(123)
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False

transform_train = transforms.Compose(
    [
        transforms.RandomHorizontalFlip(),
        transforms.RandomVerticalFlip(),
        transforms.RandomRotation(90),
        transforms.ToTensor(),
    ]
)

transform_val = transforms.Compose([transforms.ToTensor()])


class DatasetExample(Dataset):
    def __init__(self, data, img_transform):
        self.data = torch.from_numpy(data)
        self.transform = img_transform

    def __len__(
        self,
    ):
        return self.data.shape[0]

    def __getitem__(self, item):

        # img = Image.open(file_i)
        # tensor = self.transform(img)
        tensor = self.data[item][0]
        label = self.data[item][1]
        return tensor, label


if __name__ == "__main__":

    sample_x = np.linspace(0, 1, 100)
    sample_y = np.linspace(0, 1, 100) + 1
    data = np.concatenate(
        (sample_x[np.newaxis, :].T, sample_y[np.newaxis, :].T), axis=-1
    )
    np.random.shuffle(data)
    print(data[:10])

    trainset = DatasetExample(data[:60], transform_train)
    valset = DatasetExample(data[60:], transform_val)

    trainloader = DataLoader(trainset, batch_size=10, num_workers=4, shuffle=True)
    valloader = DataLoader(valset, batch_size=10, num_workers=4, shuffle=False)

    for features, labels in trainloader:
        print(features)
        print(labels)
        print(features.shape)
        print(labels.shape)
        break



import numpy as np
import torch.nn as nn
from datasets import *

d_model = 512   # 字 Embedding 的维度
d_ff = 2048     # 前向传播隐藏层维度
d_k = d_v = 64  # K(=Q), V的维度
n_layers = 6    # 有多少个encoder和decoder
n_heads = 8     # Multi-Head Attention设置为8


class PositionalEncoding(nn.Module):
    def __init__(self, d_model, dropout=0.1, max_len=5000):
        super(PositionalEncoding, self).__init__()
        self.dropout = nn.Dropout(p=dropout)
        pos_table = np.array([
            [pos / np.power(10000, 2 * i / d_model) for i in range(d_model)]
            if pos != 0 else np.zeros(d_model) for pos in range(max_len)])
        pos_table[1:, 0::2] = np.sin(pos_table[1:, 0::2])           # 字嵌入维度为偶数时
        pos_table[1:, 1::2] = np.cos(pos_table[1:, 1::2])           # 字嵌入维度为奇数时
        self.pos_table = torch.FloatTensor(pos_table).cuda()        # enc_inputs: [seq_len, d_model]

    def forward(self, enc_inputs):                                  # enc_inputs: [batch_size, seq_len, d_model]
        enc_inputs += self.pos_table[:enc_inputs.size(1), :]
        return self.dropout(enc_inputs.cuda())


def get_attn_pad_mask(seq_q, seq_k):                                # seq_q: [batch_size, seq_len] ,seq_k: [batch_size, seq_len]
    batch_size, len_q = seq_q.size()
    batch_size, len_k = seq_k.size()
    pad_attn_mask = seq_k.data.eq(0).unsqueeze(1)                   # 判断 输入那些含有P(=0),用1标记 ,[batch_size, 1, len_k]
    return pad_attn_mask.expand(batch_size, len_q, len_k)           # 扩展成多维度


def get_attn_subsequence_mask(seq):                                 # seq: [batch_size, tgt_len]
    attn_shape = [seq.size(0), seq.size(1), seq.size(1)]
    subsequence_mask = np.triu(np.ones(attn_shape), k=1)            # 生成上三角矩阵,[batch_size, tgt_len, tgt_len]
    subsequence_mask = torch.from_numpy(subsequence_mask).byte()    # [batch_size, tgt_len, tgt_len]
    return subsequence_mask


class ScaledDotProductAttention(nn.Module):
    def __init__(self):
        super(ScaledDotProductAttention, self).__init__()

    def forward(self, Q, K, V, attn_mask):                              # Q: [batch_size, n_heads, len_q, d_k]
                                                                        # K: [batch_size, n_heads, len_k, d_k]
                                                                        # V: [batch_size, n_heads, len_v(=len_k), d_v]
                                                          # attn_mask: [batch_size, n_heads, seq_len, seq_len]
        scores = torch.matmul(Q, K.transpose(-1, -2)) / np.sqrt(d_k)    # scores : [batch_size, n_heads, len_q, len_k]
        scores.masked_fill_(attn_mask, -1e9)                            # 如果时停用词P就等于 0
        attn = nn.Softmax(dim=-1)(scores)
        context = torch.matmul(attn,V)                                 # [batch_size, n_heads, len_q, d_v]
        return context, attn


class MultiHeadAttention(nn.Module):
    def __init__(self):
        super(MultiHeadAttention, self).__init__()
        self.W_Q = nn.Linear(d_model, d_k * n_heads, bias=False)
        self.W_K = nn.Linear(d_model, d_k * n_heads, bias=False)
        self.W_V = nn.Linear(d_model, d_v * n_heads, bias=False)
        self.fc = nn.Linear(n_heads * d_v, d_model, bias=False)

    def forward(self, input_Q, input_K, input_V, attn_mask):    # input_Q: [batch_size, len_q, d_model]
                                                                # input_K: [batch_size, len_k, d_model]
                                                                # input_V: [batch_size, len_v(=len_k), d_model]
                                                                # attn_mask: [batch_size, seq_len, seq_len]
        residual, batch_size = input_Q, input_Q.size(0)
        Q = self.W_Q(input_Q).view(batch_size, -1, n_heads, d_k).transpose(1, 2)    # Q: [batch_size, n_heads, len_q, d_k]
        K = self.W_K(input_K).view(batch_size, -1, n_heads, d_k).transpose(1, 2)    # K: [batch_size, n_heads, len_k, d_k]
        V = self.W_V(input_V).view(batch_size, -1, n_heads, d_v).transpose(1,
                                                                           2)       # V: [batch_size, n_heads, len_v(=len_k), d_v]
        attn_mask = attn_mask.unsqueeze(1).repeat(1, n_heads, 1,
                                                  1)                                # attn_mask : [batch_size, n_heads, seq_len, seq_len]
        context, attn = ScaledDotProductAttention()(Q, K, V, attn_mask)             # context: [batch_size, n_heads, len_q, d_v]
                                                                                    # attn: [batch_size, n_heads, len_q, len_k]
        context = context.transpose(1, 2).reshape(batch_size, -1,
                                                  n_heads * d_v)                    # context: [batch_size, len_q, n_heads * d_v]
        output = self.fc(context)                                                   # [batch_size, len_q, d_model]
        return nn.LayerNorm(d_model).cuda()(output + residual), attn


class PoswiseFeedForwardNet(nn.Module):
    def __init__(self):
        super(PoswiseFeedForwardNet, self).__init__()
        self.fc = nn.Sequential(
            nn.Linear(d_model, d_ff, bias=False),
            nn.ReLU(),
            nn.Linear(d_ff, d_model, bias=False))

    def forward(self, inputs):                                  # inputs: [batch_size, seq_len, d_model]
        residual = inputs
        output = self.fc(inputs)
        return nn.LayerNorm(d_model).cuda()(output + residual)  # [batch_size, seq_len, d_model]


class EncoderLayer(nn.Module):
    def __init__(self):
        super(EncoderLayer, self).__init__()
        self.enc_self_attn = MultiHeadAttention()                   # 多头注意力机制
        self.pos_ffn = PoswiseFeedForwardNet()                      # 前馈神经网络

    def forward(self, enc_inputs, enc_self_attn_mask):              # enc_inputs: [batch_size, src_len, d_model]
        # 输入3个enc_inputs分别与W_q、W_k、W_v相乘得到Q、K、V            # enc_self_attn_mask: [batch_size, src_len, src_len]
        enc_outputs, attn = self.enc_self_attn(enc_inputs, enc_inputs, enc_inputs,
                                                                    # enc_outputs: [batch_size, src_len, d_model],
                                               enc_self_attn_mask)  # attn: [batch_size, n_heads, src_len, src_len]
        enc_outputs = self.pos_ffn(enc_outputs)                     # enc_outputs: [batch_size, src_len, d_model]
        return enc_outputs, attn



class Encoder(nn.Module):
    def __init__(self):
        super(Encoder, self).__init__()
        self.src_emb = nn.Embedding(src_vocab_size, d_model)                     # 把字转换字向量
        self.pos_emb = PositionalEncoding(d_model)                               # 加入位置信息
        self.layers = nn.ModuleList([EncoderLayer() for _ in range(n_layers)])

    def forward(self, enc_inputs):                                               # enc_inputs: [batch_size, src_len]
        enc_outputs = self.src_emb(enc_inputs)                                   # enc_outputs: [batch_size, src_len, d_model]
        enc_outputs = self.pos_emb(enc_outputs)                                  # enc_outputs: [batch_size, src_len, d_model]
        enc_self_attn_mask = get_attn_pad_mask(enc_inputs, enc_inputs)           # enc_self_attn_mask: [batch_size, src_len, src_len]
        enc_self_attns = []
        for layer in self.layers:
            enc_outputs, enc_self_attn = layer(enc_outputs, enc_self_attn_mask)  # enc_outputs :   [batch_size, src_len, d_model],
                                                                                 # enc_self_attn : [batch_size, n_heads, src_len, src_len]
            enc_self_attns.append(enc_self_attn)
        return enc_outputs, enc_self_attns

class DecoderLayer(nn.Module):
    def __init__(self):
        super(DecoderLayer, self).__init__()
        self.dec_self_attn = MultiHeadAttention()
        self.dec_enc_attn = MultiHeadAttention()
        self.pos_ffn = PoswiseFeedForwardNet()

    def forward(self, dec_inputs, enc_outputs, dec_self_attn_mask,
                dec_enc_attn_mask):                                             # dec_inputs: [batch_size, tgt_len, d_model]
                                                                                # enc_outputs: [batch_size, src_len, d_model]
                                                                                # dec_self_attn_mask: [batch_size, tgt_len, tgt_len]
                                                                                # dec_enc_attn_mask: [batch_size, tgt_len, src_len]
        dec_outputs, dec_self_attn = self.dec_self_attn(dec_inputs, dec_inputs,
                                                        dec_inputs,
                                                        dec_self_attn_mask)     # dec_outputs: [batch_size, tgt_len, d_model]
                                                                                # dec_self_attn: [batch_size, n_heads, tgt_len, tgt_len]
        dec_outputs, dec_enc_attn = self.dec_enc_attn(dec_outputs, enc_outputs,
                                                      enc_outputs,
                                                      dec_enc_attn_mask)        # dec_outputs: [batch_size, tgt_len, d_model]
                                                                                # dec_enc_attn: [batch_size, h_heads, tgt_len, src_len]
        dec_outputs = self.pos_ffn(dec_outputs)                                 # dec_outputs: [batch_size, tgt_len, d_model]
        return dec_outputs, dec_self_attn, dec_enc_attn


class Decoder(nn.Module):
    def __init__(self):
        super(Decoder, self).__init__()
        self.tgt_emb = nn.Embedding(tgt_vocab_size, d_model)
        self.pos_emb = PositionalEncoding(d_model)
        self.layers = nn.ModuleList([DecoderLayer() for _ in range(n_layers)])

    def forward(self, dec_inputs, enc_inputs, enc_outputs):                         # dec_inputs: [batch_size, tgt_len]
                                                                                    # enc_intpus: [batch_size, src_len]
                                                                                    # enc_outputs: [batsh_size, src_len, d_model]
        dec_outputs = self.tgt_emb(dec_inputs)                                      # [batch_size, tgt_len, d_model]
        dec_outputs = self.pos_emb(dec_outputs).cuda()                              # [batch_size, tgt_len, d_model]
        dec_self_attn_pad_mask = get_attn_pad_mask(dec_inputs, dec_inputs).cuda()   # [batch_size, tgt_len, tgt_len]
        dec_self_attn_subsequence_mask = get_attn_subsequence_mask(dec_inputs).cuda()  # [batch_size, tgt_len, tgt_len]
        dec_self_attn_mask = torch.gt((dec_self_attn_pad_mask +
                                       dec_self_attn_subsequence_mask), 0).cuda()   # [batch_size, tgt_len, tgt_len]
        dec_enc_attn_mask = get_attn_pad_mask(dec_inputs, enc_inputs)               # [batc_size, tgt_len, src_len]
        # print('dec',dec_self_attn_mask)
        dec_self_attns, dec_enc_attns = [], []
        for layer in self.layers:                                                   # dec_outputs: [batch_size, tgt_len, d_model]
                                                                                    # dec_self_attn: [batch_size, n_heads, tgt_len, tgt_len]
                                                                                    # dec_enc_attn: [batch_size, h_heads, tgt_len, src_len]
            dec_outputs, dec_self_attn, dec_enc_attn = layer(dec_outputs, enc_outputs, dec_self_attn_mask,
                                                             dec_enc_attn_mask)
            dec_self_attns.append(dec_self_attn)
            dec_enc_attns.append(dec_enc_attn)
        return dec_outputs, dec_self_attns, dec_enc_attns


class Transformer(nn.Module):
    def __init__(self):
        super(Transformer, self).__init__()
        self.Encoder = Encoder().cuda()
        self.Decoder = Decoder().cuda()
        self.projection = nn.Linear(d_model, tgt_vocab_size, bias=False).cuda()

    def forward(self, enc_inputs, dec_inputs):                          # enc_inputs: [batch_size, src_len]
                                                                        # dec_inputs: [batch_size, tgt_len]
        enc_outputs, enc_self_attns = self.Encoder(enc_inputs)          # enc_outputs: [batch_size, src_len, d_model],
                                                                        # enc_self_attns: [n_layers, batch_size, n_heads, src_len, src_len]
        dec_outputs, dec_self_attns, dec_enc_attns = self.Decoder(
            dec_inputs, enc_inputs, enc_outputs)                        # dec_outpus    : [batch_size, tgt_len, d_model],
                                                                        # dec_self_attns: [n_layers, batch_size, n_heads, tgt_len, tgt_len],
                                                                        # dec_enc_attn  : [n_layers, batch_size, tgt_len, src_len]
        dec_logits = self.projection(dec_outputs)                       # dec_logits: [batch_size, tgt_len, tgt_vocab_size]
        return dec_logits.view(-1, dec_logits.size(-1)), enc_self_attns, dec_self_attns, dec_enc_attns


"""
@File    :   assistant_v2.py
@Time    :   2024/07/04 12:21:43
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import Callable, Optional, Union, List
import multiprocessing as mp
import os
import os.path as osp
import time
import copy
import argparse

import pynput
import cv2
import pyaudio
import pyttsx3
import numpy as np
import torch
from PIL import ImageGrab
# from faster_whisper import WhisperModel
from transformers import AutoModelForSpeechSeq2Seq, AutoProcessor, pipeline

import qianfan

parser = argparse.ArgumentParser("assistant")
parser.add_argument(
    "-a",
    "--access_key",
    required=True,
)
parser.add_argument(
    "-s",
    "--secret_key",
    required=True,
)
parser.add_argument(
    "-m",
    "--model",
    default="ERNIE-Bot-turbo",  # ERNIE-4.0-8K ChatLaw
    help="ERNIE-4.0-8K, ChatLaw, ERNIE-Bot-turbo...",
)

parser.add_argument(
    "-l",
    "--record_src_language",
    default="chinese", #"zh",
    help="record src language",
)

parser.add_argument(
    "-sp",
    "--is_speak",
    action="store_true",
    help="assistant speak",
)


args = parser.parse_args()

os.environ["QIANFAN_ACCESS_KEY"] = args.access_key
os.environ["QIANFAN_SECRET_KEY"] = args.secret_key


INT16_MAX_ABS_VALUE = 32768.0
CHUNK = 512
FORMAT = pyaudio.paInt16
CHANNELS = 1
RATE = 16000
unit = RATE // CHUNK

# model_id = "large-v3"
# device = "cuda" if torch.cuda.is_available() else "cpu"
# stt_model = WhisperModel(model_id, device=device, compute_type="float16")

model_id = "openai/whisper-large-v3-turbo"
device = "cuda" if torch.cuda.is_available() else "cpu"
torch_dtype = torch.float16 if torch.cuda.is_available() else torch.float32
stt_model = AutoModelForSpeechSeq2Seq.from_pretrained(
    model_id, torch_dtype=torch_dtype, low_cpu_mem_usage=True
)
stt_model.to(device)

processor = AutoProcessor.from_pretrained(model_id)

pipe = pipeline(
    "automatic-speech-recognition",
    model=stt_model,
    tokenizer=processor.tokenizer,
    feature_extractor=processor.feature_extractor,
    chunk_length_s=30,
    batch_size=16,  # batch size for inference - set based on your device
    torch_dtype=torch_dtype,
    device=device,
)

generate_kwargs={"language": "chinese",    
                 "logprob_threshold": -1.0,
                 "no_speech_threshold": 0.6,
                 "return_timestamps": False,}


def on_press_wrapper(signal: mp.Value) -> Callable:
    def on_press(key: pynput.keyboard.Key) -> Optional[bool]:
        nonlocal signal
        if key == pynput.keyboard.Key.esc:
            signal.value = -1
            return False
        elif hasattr(key, "char"):
            if key.char == "f":
                # pyttsx3.speak('Hello, World!我们的花朵是花园')
                print("start record...")
                signal.value = 1
            if key.char == "s":
                print("end record...")
                signal.value = 0

    return on_press
    # print("Key {0} pressed".format(key))


def screenshot():
    # 获取屏幕截图
    screenshot = ImageGrab.grab()


def camera():
    # 打开摄像头
    ...


def execute_instructions(): ...


def kb_listener(signal: mp.Value) -> None:
    listener = pynput.keyboard.Listener(on_press=on_press_wrapper(signal))
    listener.start()
    listener.join()


if __name__ == "__main__":
    chat_comp = qianfan.ChatCompletion()
    msgs = qianfan.Messages()

    p = pyaudio.PyAudio()
    # 打开麦克风流
    stream = p.open(
        format=FORMAT,
        channels=CHANNELS,
        rate=RATE,
        input=True,
        output=True,
        frames_per_buffer=CHUNK,
    )
    audio_collector = []

    signal = mp.Value("i", 0)
    listener = mp.Process(target=kb_listener, args=(signal,))
    listener.daemon = True
    print("按下F开始录音，按下S暂停录音，按下ESC退出程序")
    print("ready")
    listener.start()

    while True:
        if signal.value == 1:
            audio_data = stream.read(CHUNK)
            audio_collector.append(audio_data)
        elif signal.value == 0 and len(audio_collector) > 0:
            audio_array = np.frombuffer(b"".join(audio_collector), dtype=np.int16)
            audio_array = audio_array.astype(np.float32) / INT16_MAX_ABS_VALUE

            # segments, info = stt_model.transcribe(
            #     audio_array,
            #     language=args.record_src_language,
            #     beam_size=5,
            #     log_prob_threshold=-0.5,
            # )
            segments = pipe(audio_array, generate_kwargs=generate_kwargs)


            text = ""
            # for segment in segments:
            #     if segment.text != "":
            #         text += segment.text
            if segments['text'] != "":
                text += segments['text']
            print("[master]:", text)
            msgs.append(text)
            resp = chat_comp.do(model=args.model, messages=msgs)
            print("[assistant]:", resp["result"])
            if args.is_speak:
                pyttsx3.speak(resp["result"])
            msgs.append(resp)
            audio_collector = []

        elif signal.value == -1:
            break
        else:
            pass
        # time.sleep(0.2)

    listener.join()

    stream.stop_stream()
    stream.close()
    p.terminate()



# 图像配准

import numpy as np
import cv2
from PIL import ImageGrab
from sklearn.cluster import DBSCAN


def image_registration(
    query_img_path: str, query_img_scale: float = 1.0, threshold: float = 1.0, min_samples: int =4,is_show: bool = True
):

    # 读取拼接图片
    imageA = cv2.imread(query_img_path)
    imageA = cv2.resize(imageA, None, fx=query_img_scale, fy=query_img_scale)

    imageB = ImageGrab.grab()
    imageB = cv2.cvtColor(np.array(imageB), cv2.COLOR_RGB2BGR)

    # 读取图像
    img1 = imageA
    img2 = imageB

    # 初始化SIFT检测器
    sift = cv2.SIFT_create()

    # 使用SIFT找到关键点和描述符
    kp1, descriptors1 = sift.detectAndCompute(img1, None)
    kp2, descriptors2 = sift.detectAndCompute(img2, None)

    # 创建匹配器
    bf = cv2.BFMatcher()
    matches = bf.knnMatch(descriptors1, descriptors2, k=2)##

    # 应用比率测试
    good = []
    n_good = []
    for m, n in matches:
        if m.distance < 0.75 * n.distance:
            good.append(m)
            n_good.append(n)
    if not good:
        print('没有匹配点')
        return None
    else:
        # 绘制图像
        pts2 = np.float32([kp2[n.trainIdx].pt for n in n_good]).reshape(-1, 2)
        dbscan = DBSCAN(eps=threshold, min_samples=min_samples)
 
        # 对数据集进行聚类
        labels = dbscan.fit_predict(pts2)
        print('labels:',labels)
        #is_core_sample = dbscan.core_sample_indices_
        # 剔除离群点，保留标签为-1的点
        # pts2 = pts2[~outliers.any(axis=1)]
        filtered_good = []
        for idx, (m, n) in enumerate(zip(good, n_good)):
            if labels[idx]>=0:
                filtered_good.append([m])


        img_matches = cv2.drawMatchesKnn(
            img1, kp1, img2, kp2, filtered_good, None, flags=2
        )
        if is_show:
            cv2.namedWindow("Matches", cv2.WINDOW_NORMAL)
            cv2.imshow("Matches", img_matches)
            cv2.waitKey(0)
            cv2.destroyAllWindows()

        if len(filtered_good) <= 4:
            print('匹配点小于5')
            return None
        else:
            pts1 = np.float32([kp1[m[0].queryIdx].pt for m in filtered_good]).reshape(
                -1, 1, 2
            )
            pts2 = np.float32([kp2[m[0].trainIdx].pt for m in filtered_good]).reshape(
                -1, 1, 2
            )

            min_x1 = np.min(pts1[:, :, 0])
            max_x1 = np.max(pts1[:, :, 0])
            min_y1 = np.min(pts1[:, :, 1])
            max_y1 = np.max(pts1[:, :, 1])
            min_x2 = np.min(pts2[:, :, 0])
            max_x2 = np.max(pts2[:, :, 0])
            min_y2 = np.min(pts2[:, :, 1])
            max_y2 = np.max(pts2[:, :, 1])

            scale_x = (max_x2 - min_x2) / (max_x1 - min_x1)
            scale_y = (max_y2 - min_y2) / (max_y1 - min_y1)
            pts1_norm=pts1-[min_x1,min_y1]
            pts2_norm=pts2-[min_x2,min_y2]
            H, masked = cv2.findHomography(pts1_norm, pts2_norm, cv2.RANSAC, 5.0)

            # 使用单应性矩阵变换查询图像
            height, width, _ = img1.shape
            new_width = int(width * scale_x)
            new_height = int(height * scale_y)

            img1_transformed = cv2.warpPerspective(img1, H, (new_width,new_height))
            #img1_transformed = cv2.resize(img1, (new_width, new_height))
            #print(img1_transformed.shape)
            if is_show:
                cv2.namedWindow("Matches", cv2.WINDOW_NORMAL)
                cv2.imshow("Matches", img1_transformed)
                cv2.waitKey(0)
                cv2.destroyAllWindows()

            # 将变换后的查询图像定位到全景图像中
            try:
                img2[
                    int(min_y2-min_y1*scale_y) : int(min_y2-min_y1* scale_y) + new_height,
                    int(min_x2-min_x1*scale_x) : int(min_x2-min_x1*scale_x) + new_width,
                ] = img1_transformed
            except:
                print("out of range，可能有异常点导致图片切片过大")
                return None

            # 显示图像
            else:
                if is_show:
                    cv2.namedWindow("Matches", cv2.WINDOW_NORMAL)
                    cv2.imshow("Matches", img2)
                    cv2.waitKey(0)
                    cv2.destroyAllWindows()


if __name__ == "__main__":
    image_registration(query_img_path=r"D:\work\ComfyUI\output\ComfyUI_00070.png", query_img_scale=0.2, threshold=22.0, min_samples=3, is_show=True)



# ref: https://juejin.cn/post/7374055681890795555

from typing import Callable, Any, Dict
import random
import time


__all__ = ["user_login"]  # 装饰器

users_db = {"user-a": "123password", "user-b": "456password", "user-c": "789password"}

# 该字典用于存放各个用户登录密码错误的次数
login_counter: Dict = {}


def generate_captcha() -> str:
    """生成四位数字的验证码"""
    return str(random.randint(1000, 9999))


def verify_captcha(captcha_input: str, captcha_actual: str) -> bool:
    """验证用户输入的验证码是否正确"""
    return captcha_input == captcha_actual


def user_login(func: Callable) -> Callable:
    def wrapper(*args, **kwargs) -> Any:
        while True:
            """输入用户登录信息"""
            username = input("请输入用户名：")
            password = input("请输入密码：")

            # 生成验证码
            captcha = generate_captcha()
            print(f"验证码是：{captcha}")
            captcha_input = input("请输入验证码：")

            if username in users_db:

                if not verify_captcha(captcha_input, captcha):
                    print("验证码错误！")
                    continue
                # 密码验证
                if users_db[username] == password:
                    print("登录成功！")
                    return func(*args, **kwargs)
                    break  # 登录成功，退出循环
                else:  # 密码错误次数记录与锁定账号
                    if username in login_counter:
                        login_counter[username] += 1
                    else:
                        login_counter[username] = 1

                    if login_counter[username] >= 3:
                        print("密码错误次数过多，账号已被锁定！")
                        break  # 输错三次密码，锁定账号，退出循环
                    else:
                        print("密码错误！请重新输入！")
            else:
                print("用户名不存在，请重新输入！")

    return wrapper


if __name__ == "__main__":

    @user_login
    def funcion(x, y):
        return x + y

    print(funcion(6, 9))



"""
@File    :   pickle_op.py
@Time    :   2024/07/02 23:10:29
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if "." not in sys.path:
    sys.path.append(".")

from typing import Union, Dict, List, Tuple, Any
import os
import pickle

from rlmc.fileop.abstract_file import AbstractFile

__all__ = ["Pickle"]


class Pickle(AbstractFile):
    def __init__(self, file_path: str = "", mode: str = "rb") -> None:
        self.file_path = file_path
        self.mode = mode

        if file_path != "":
            self.data = self.read(file_path)

    def __enter__(self):
        self.file = open(self.file_path, self.mode)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.file.close()
        if exc_type != None:
            print(exc_type, exc_val, exc_tb)

    def read(self, file_path: str = "") -> Any:
        with open(file_path, self.mode) as f:
            data = pickle.load(f)
        return data

    def write(self, data: Any, file_path: str, mode: str = "wb") -> None:
        with open(file_path, mode) as f:
            pickle.dump(data, f)



# -*- coding:utf-8 -*-
# @Time    : 2023/1/13 23:19


import torch
import torch.utils.data as Data

# Encoder_input    Decoder_input        Decoder_output
sentences = [['我 是 学 生 P', 'S I am a student', 'I am a student E'],         # S: 开始符号
             ['我 喜 欢 学 习', 'S I like learning P', 'I like learning P E'],  # E: 结束符号
             ['我 是 男 生 P', 'S I am a boy', 'I am a boy E']]                 # P: 占位符号，如果当前句子不足固定长度用P占位

src_vocab = {'P': 0, '我': 1, '是': 2, '学': 3, '生': 4, '喜': 5, '欢': 6, '习': 7, '男': 8}  # 词源字典  字：索引
src_idx2word = {src_vocab[key]: key for key in src_vocab}
src_vocab_size = len(src_vocab)  # 字典字的个数
tgt_vocab = {'P': 0, 'S': 1, 'E': 2, 'I': 3, 'am': 4, 'a': 5, 'student': 6, 'like': 7, 'learning': 8, 'boy': 9}
idx2word = {tgt_vocab[key]: key for key in tgt_vocab}   # 把目标字典转换成 索引：字的形式
tgt_vocab_size = len(tgt_vocab)                         # 目标字典尺寸
src_len = len(sentences[0][0].split(" "))               # Encoder输入的最大长度
tgt_len = len(sentences[0][1].split(" "))               # Decoder输入输出最大长度


# 把sentences 转换成字典索引
def make_data():
    enc_inputs, dec_inputs, dec_outputs = [], [], []
    for i in range(len(sentences)):
        enc_input = [[src_vocab[n] for n in sentences[i][0].split()]]
        dec_input = [[tgt_vocab[n] for n in sentences[i][1].split()]]
        dec_output = [[tgt_vocab[n] for n in sentences[i][2].split()]]
        enc_inputs.extend(enc_input)
        dec_inputs.extend(dec_input)
        dec_outputs.extend(dec_output)
        print('in',torch.LongTensor(enc_inputs))
        print('decin', torch.LongTensor(dec_inputs))
        print('out',torch.LongTensor(enc_inputs).shape,torch.LongTensor(dec_outputs).shape)
    return torch.LongTensor(enc_inputs), torch.LongTensor(dec_inputs), torch.LongTensor(dec_outputs)


# 自定义数据集函数
class MyDataSet(Data.Dataset):
    def __init__(self, enc_inputs, dec_inputs, dec_outputs):
        super(MyDataSet, self).__init__()
        self.enc_inputs = enc_inputs
        self.dec_inputs = dec_inputs
        self.dec_outputs = dec_outputs

    def __len__(self) -> int:
        return self.enc_inputs.shape[0]

    def __getitem__(self, idx):
        return self.enc_inputs[idx], self.dec_inputs[idx], self.dec_outputs[idx]


import sys

if "." not in sys.path:
    sys.path.append(".")

from rlmc import cfg
from rlmc import reg

def test_yaml():
    file_processor = reg.file_processor
    yamlobj = file_processor("rlmc/configs/common_urls.yaml")
    yaml_content = yamlobj.data
    assert yaml_content != "", "yaml content is empty"
    print(yaml_content)

def test_cfg():
    assert cfg.source.cuda != "", "cfg content is empty"
    print(cfg.source.cuda)


if __name__ == "__main__":
    # MsDownload=reg['AutodlDownload']
    # repo_id="tzwm/StableDiffusion-others/res101.pth"
    # ms_download=MsDownload(repo_id)
    # ms_download.run()
    file_processor = reg["file_processor"]
    yamlobj = file_processor("rlmc/configs/common_urls.yaml")
    yaml_content = yamlobj.data
    print(yaml_content)
    # print(cfg['source']['cuda'])
    print(cfg.source.cuda)


"""
@File    :   multithread.py
@Time    :   2024/06/21 00:19:26
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if '.' not in sys.path: sys.path.append(".")

from typing import Any, Union, Callable, Iterable, List, Dict, Tuple
import concurrent.futures as cf

import time

from tqdm import tqdm
from rlmc.utils.logger import Logger


__all__ = ["MultiThread"]

logger = Logger(__name__, level=Logger.DEBUG)


class MultiThread:

    def __init__(self, func: Callable, worker: int) -> None:
        self.func = func
        self.worker = worker

    def unpack_args(self, idx_args: Tuple[int, Union[Dict, Tuple]]) -> Tuple[int, Any]:
        args = idx_args[1]
        idx = idx_args[0]
        if isinstance(args, tuple):
            return idx, self.func(*args)
        elif isinstance(args, dict):
            return idx, self.func(**args)
        else:
            raise TypeError("args must be tuple or dict")

    def run(self, tasklist: Iterable[Any]) -> List:
        collector = []
        with cf.ThreadPoolExecutor(max_workers=self.worker) as executor:
            future_to_result = {
                executor.submit(self.unpack_args, (idx, i)): (idx, i)
                for idx, i in enumerate(tasklist)
            }

        for future in cf.as_completed(future_to_result):
            result = future.result()
            print(f"Task {future_to_result[future]} returned {result}")
            collector.append(result)
        return collector


def test_unit(a, b):
    return a + b, b


def test_unit_no_return(a, b):
    print(a + b, b)


if __name__ == "__main__":

    a = [i for i in range(100)]
    b = [i + 1 for i in range(100)]
    task_list = list(zip(a, b))

    task_list = []
    for i in range(10):
        task_list.append({"a": i, "b": i + 1})
    # print(task_list)

    mut = MultiThread(test_unit, 4)
    res = mut.run(task_list)
    print(res)



"""
@File    :   coroutine.py
@Time    :   2024/06/21 09:02:18
@Author  :   RayLam
@Contact :   1027196450@qq.com
"""

import sys

if '.' not in sys.path: sys.path.append(".")

import time
from rlmc.utils.logger import Logger


__all__ = ["Abstract_ManMachineChat"]

logger = Logger(__name__, level=Logger.DEBUG)


from abc import ABCMeta, abstractmethod


class Abstract_ManMachineChat(metaclass=ABCMeta):
    @abstractmethod
    def person(self):
        msg = ""
        while True:
            msg = yield msg
            logger.info(f"person 1:{msg}")
            time.sleep(1)

    @abstractmethod
    def computer(self):
        msg = ""
        while True:
            msg = yield msg
            logger.info(f"computer 2:{msg},i'm computer")
            time.sleep(1)

    @abstractmethod
    def runchat(self):
        task1 = self.person()
        task2 = self.computer()
        next(task1)
        next(task2)
        while True:
            msg1 = task1.send(input("person 1 input:"))
            msg2 = task2.send(msg1)
            print(msg2)


class ManMachineChatExample(Abstract_ManMachineChat):
    def person(self):
        msg = ""
        while True:
            msg = yield msg
            logger.debug(f"person 1:{msg}")
            time.sleep(1)

    def computer(self):
        msg = ""
        while True:
            msg = yield msg + 'yield1'
            logger.debug(f"computer 21:{msg},i'm computer")
            msg = yield msg + 'yield2'
            logger.debug(f"computer 22:{msg}")
            time.sleep(1)

    def runchat(self):
        task1 = self.person()
        task2 = self.computer()
        next(task1)
        next(task2)
        while True:
            msg1 = task1.send(input("person 1 input:"))
            msg2 = task2.send(msg1)
            print(msg2)


def task1():
    msg = ""
    epoch=0
    while True:
        msg = yield f'task_yield1:{epoch}' 
        print(f'task_{msg}_1')
        msg = yield f'task_yield2:{epoch}' 
        print(f'task_{msg}_2')
        print('task_n:',epoch)
        epoch+=1
        time.sleep(1)
        if epoch>6:   # 最好在task内控制轮数
            break


def how_to_understand_yield():
    t1= task1()
    next(t1)
    counter=0
    while True:
        try:
            msg1 = t1.send(input("person 1 input:"))
        except:
            break
        else:
            print('how_counter:',counter)
            print('how_return_msg:',msg1)
            counter+=1

if __name__ == "__main__":
    # chat = ManMachineChatExample()
    # chat.runchat()

    how_to_understand_yield()



from setuptools import setup
from Cython.Build import cythonize
from distutils.core import Extension

setup(
    # ext_modules=cythonize(
    #                   "rltools/del_jupyter_temp.pyx"
    #         )
    ext_modules=[Extension("rltools.del_jupyter_temp", ["rltools/del_jupyter_temp.c"])]
)



import sys

if "." not in sys.path:
    sys.path.append(".")

import os
import os.path as osp
import json
from rlmc.fileop import file_processor

__all__ = [
    "user_setting",
    "model_download_urls",
    "dataset_download_urls",
    "common_urls",
]

config_root = osp.dirname(__file__)


user_setting = file_processor(osp.join(config_root, "user_setting.yaml")).data
common_urls = file_processor(osp.join(config_root, "common_urls.yaml")).data
model_download_urls = file_processor(
    osp.join(config_root, "models/model_download_urls.yaml")
).data
dataset_download_urls = file_processor(
    osp.join(config_root, "datasets/dataset_download_urls.yaml")
).data
# print(user_setting)




import tensorflow.compat.v1 as tf1

tf1.disable_v2_behavior()


ckpt_path = r"D:\work\jupyfile\trainmodel\modl-1"
reader = tf1.train.NewCheckpointReader(ckpt_path)

saver = tf1.train.import_meta_graph(ckpt_path + ".meta", clear_devices=True)
graph = tf1.get_default_graph()
with tf1.Session(graph=graph) as sess:
    sess.run(tf1.global_variables_initializer())
    saver.restore(sess, ckpt_path)
    print(sess.graph.collections)
    print(sess.graph.get_collection("variables"))


# 获取 变量名: 形状
vars = reader.get_variable_to_shape_map()
for k in sorted(vars):
    print(k, vars[k])

# 获取 变量名: 类型
vars = reader.get_variable_to_dtype_map()
for k in sorted(vars):
    print(k, vars[k])

# 获取张量的值
value = reader.get_tensor("bias")
print(value)



import sys

if "." not in sys.path:
    sys.path.append(".")

import argparse
import platform
import os
import os.path as osp
from rlmc.resource import condarc, pip

parser = argparse.ArgumentParser("manage source")
parser.add_argument("-s", "--source", default="pip", help="pip, conda, apt")
args = parser.parse_args()

codename = (
    os.popen("cat /etc/os-release | grep UBUNTU_CODENAME").read().strip().split("=")[1]
)
apt = f"deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename} main restricted\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename}-updates main restricted\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename} universe\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename}-updates universe\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename} multiverse\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename}-updates multiverse\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename}-backports main restricted universe multiverse\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename}-security main restricted\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename}-security universe\n \
deb http://mirrors.tuna.tsinghua.edu.cn/ubuntu/ {codename}-security multiverse"


def manage_source() -> None:
    if platform.system() == "Windows":
        pip_src = osp.join(osp.expanduser("~"), "pip/pip.ini")
    elif platform.system() == "Linux":
        pip_src = osp.join(osp.expanduser("~"), ".pip/pip.conf")

    conda_src = osp.join(osp.expanduser("~"), ".condarc")
    if args.source == "pip":
        if osp.exists(pip_src):
            print("####################### pip ori src #######################\n")
            with open(pip_src, "r") as f:
                print(f.read())

        command = input("change pip source? (y/n):")
        if command == "y":
            os.makedirs(osp.dirname(pip_src), exist_ok=True)
            with open(pip_src, "w") as f:
                f.write(pip)
    elif args.source == "conda":
        if osp.exists(conda_src):
            print("####################### conda ori src #######################\n")
            with open(conda_src, "r") as f:
                print(f.read())

        command = input("change conda source? (y/n):")
        if command == "y":
            with open(conda_src, "w") as f:
                f.write(condarc)

    elif args.source == "apt":
        if platform.system() == "Linux":
            apt_src = "/etc/apt/sources.list"
            if osp.exists(apt_src):
                print("####################### apt ori src #######################\n")
                with open(apt_src, "r") as f:
                    print(f.read())
            command = input("change apt source? (y/n):")
            if command == "y":
                if not osp.exists("/etc/apt/sources.list.bak"):
                    os.system(f"cp {apt_src} /etc/apt/sources.list.bak")
                with open(apt_src, "w") as f:
                    f.write(apt)


if __name__ == "__main__":
    manage_source()



import sys

if "." not in sys.path:
    sys.path.append(".")

__all__ = ["datasets"]

from rlmc.data.semantic_segmentation import SemanticSegmentationDataset

datasets = {
    "seg": SemanticSegmentationDataset,
}



import asyncio
import time
# a blocking io-bound task
def blocking_task():
    # report a message
    print('Task starting')
    # block for a while
    time.sleep(2)
    # report a message
    print('Task done')
    return 'done'
# main coroutine
async def main():
    # report a message
    print('Main running the blocking task')
    # create a coroutine for  the blocking task
    coro = asyncio.to_thread(blocking_task)
    print('coro',type(coro))
    # schedule the task
    task = asyncio.create_task(coro)
    print('taskk',type(task))
    # report a message
    print('Main doing other things')
    # allow the scheduled task to start
    await asyncio.sleep(0)
    # await the task
    return await task
# run the asyncio program
asyncio.run(main())
print(type(asyncio.run(main())))


import argparse
import os
import os.path as osp
import glob
import re

from tqdm import tqdm

parser = argparse.ArgumentParser("findtool")
parser.add_argument("-r", "--root", required=True, help="scan the path")
parser.add_argument("-n", "--name", default="*", help="name except extension")
parser.add_argument(
    "-e", "--exts", nargs="+", help="extension, like .py .conf if empty, fill * "
)
parser.add_argument(
    "-rc", "--recursive", action="store_true", help="glob files not recursive"
)
parser.add_argument(
    "-d", "--dir", action="store_true", help="not find the files, find dir"
)
parser.add_argument(
    "-rg",
    "--regex",
    action="store_true",
    help="is use regex to find content",
)
parser.add_argument(
    "-c", "--content", default="", help="find the content,if empty,find all files"
)
parser.add_argument("--encoding", default="utf-8")
args = parser.parse_args()


def find_string_in_file(file_path: str, search_string: str) -> bool:
    with open(file_path, "r", encoding=args.encoding) as file:
        for line in file.readlines():
            if args.regex:
                if re.search(search_string, line):
                    return True
            else:
                if search_string in line:
                    return True
    return False


def find() -> None:
    files = []
    if args.exts:
        for ext in args.exts:
            if args.recursive:
                files.extend(
                    glob.glob(
                        osp.join(args.root, "**", args.name + ext), recursive=True
                    )
                )
            else:
                files.extend(
                    glob.glob(osp.join(args.root, args.name + ext), recursive=False)
                )
    else:
        if not args.dir:
            if args.recursive:
                files.extend(
                    glob.glob(
                        osp.join(args.root, "**", args.name + ".*"), recursive=True
                    )
                )
            else:
                files.extend(
                    glob.glob(osp.join(args.root, args.name + ".*"), recursive=False)
                )
        else:
            if args.recursive:
                files.extend(
                    glob.glob(
                        osp.join(args.root, "**", args.name + "/"), recursive=True
                    )
                )
            else:
                files.extend(
                    glob.glob(osp.join(args.root, args.name + "/"), recursive=False)
                )

    if args.dir:
        files = [f for f in files if osp.isdir(f)]
    else:
        files = [f for f in files if osp.isfile(f)]
    print(files)
    print(len(files))

    if args.content != "":
        collector = []
        for file in tqdm(files):
            try:
                is_found = find_string_in_file(file, args.content)
            except UnicodeDecodeError as ue:
                print(f"\nskip:UnicodeDecodeError processing file {file}: {ue}")
                is_found = False
            except Exception as e:
                print(f"\nskip:Error processing file {file}: {e}")
                is_found = False
            if is_found:
                collector.append(file)

        if collector:
            print("result:", collector)
        else:
            print("No files match")


if __name__ == "__main__":
    find()
